from .documentation import DocumentationBridgerViewSetConfig
from .fields import FieldsBridgerViewSetConfig
from .identifier import IdentifierBridgerViewSetConfig
from .pagination import PaginationBridgerViewSetConfig
from .pk import PKBridgerViewSetConfig
from .type import TypeBridgerViewSetConfig
from .filter_fields import FilterFieldsBridgerViewSetConfig
from .search_fields import SearchFieldsBridgerViewSetConfig
from .ordering_fields import OrderingFieldsBridgerViewSetConfig
